package ap;

import ap.parameters.GoalSettings
import ap.parser.PlainTranslator
import ap.terfor.Formula
import ap.terfor.conjunctions.{Conjunction, Quantifier}
import ap.proof.{ModelSearchProver, ExhaustiveProver, ConstraintSimplifier, ProofTree}
import ap.proof.goal.Goal

abstract class AbstractFileProver(reader : java.io.Reader, output : Boolean,
                                  timeout : Int, userDefStoppingCond : => Boolean,
                                  goalSettings : GoalSettings) {

  protected def println(str : => String) : unit = (if (output) Predef.println(str))
  
  val (formula, eliminatedConstants, order) = {
    val input = PlainTranslator(reader)
    reader.close
    input
  }

  protected def findCounterModel : ModelSearchProver.Result = {
    println("Constructing countermodel ...")
    val timeBefore = System.currentTimeMillis
    def stoppingCond : Boolean =
      (System.currentTimeMillis - timeBefore > timeout) || userDefStoppingCond
        
    ModelSearchProver(formula, order, stoppingCond, goalSettings)
  }
  
  protected def findModel(f : Conjunction) : Conjunction =
    ModelSearchProver(f.negate, f.order)
  
  protected def constructProofTree(simplifier : ConstraintSimplifier) : ProofTree = {
    // explicitly quantify all universal variables
    
    val closedFor =
      Conjunction.quantify(Quantifier.ALL, eliminatedConstants, formula, order)
    
    println("Proving ...")
    
    val timeBefore = System.currentTimeMillis
    def stoppingCond(tree : ProofTree) : Boolean =
      (System.currentTimeMillis - timeBefore > timeout) ||
      userDefStoppingCond
      
    new ExhaustiveProver(simplifier, stoppingCond _, true, goalSettings)(closedFor, order)
  }

}
